/* vim: set sw=2 ts=2 sts=2 noet: */

/* For GUADEC2009 WebKitGTK+ presentation: July 2009
   Copyright © 2009 Christian Dywan <christian@twotoasts.de> */

/* Modified for EncuentroLinux 2009: 22nd - 24th October
   Copyright © 2009 Diego Escalante Urrelo <diegoe@gnome.org> */

/* This code is licensed under the terms of the expat license */

/* Build with:
   gcc -o wk wk.c $(pkg-config --cflags --libs webkit-1.0) -Wall -O1 -g */

#include <webkit/webkit.h>
#include <JavaScriptCore/JavaScript.h>

#define UI_FILE "encuentro.ui"
#define FILE_START "file:///home/diego/docs/presentaciones/EL2009: webkit y desarrollo desktop/start.html"

static void
home_clicked_cb (GtkToolButton  *toolbutton,
                  WebKitWebView  *web_view)
{
  webkit_web_view_load_uri (web_view, FILE_START);
}

static void
quit_clicked_cb (GtkToolButton *toolbutton,
                 gpointer        user_data)
{
  gtk_main_quit ();
}

static gboolean
window_delete_event_cb (GtkWidget  *widget,
                        GdkEvent   *event,
                        gpointer    user_data)
{
  gtk_main_quit ();

  return FALSE;
}

static void
zoomscale_value_changed_cb (GtkRange        *range,
                            WebKitWebView   *web_view)
{
  g_object_set (G_OBJECT (web_view),
                "zoom-level", gtk_range_get_value (range),
                NULL);
}

static void
web_plugin_clicked_cb (GtkButton     *button,
                       WebKitWebView *web_view)
{
  char *script;

  script = "document.body.style.backgroundColor = \"#000000\";";
  webkit_web_view_execute_script (web_view, script);

  script = "document.body.style.color = \"#ffffff\";";
  webkit_web_view_execute_script (web_view, script);
}

static GtkWidget*
web_view_create_plugin_widget_cb (GtkWidget*   web_view,
                                  const gchar* mime_type,
                                  const gchar* uri,
                                  GHashTable*  param,
                                  gpointer     userdata)
{
  if (g_str_equal (mime_type, "application/x-gtk-encuentrolinux"))
  {
    GtkWidget *widget;

    widget = gtk_button_new_from_stock (GTK_STOCK_APPLY);
    g_signal_connect (widget, "clicked",
                      G_CALLBACK (web_plugin_clicked_cb), web_view);

    gtk_widget_show (widget);

    return widget;
  }

  return NULL;
}

GtkWidget *window = NULL;

static JSValueRef
hola_encuentro_cb (JSContextRef     js_context,
                   JSObjectRef      js_function,
                   JSObjectRef      js_this,
                   size_t           argument_count,
                   const JSValueRef js_arguments[],
                   JSValueRef*      js_exception)
{
  JSValueRef js_value = JSValueMakeNull (js_context);

  if (argument_count == 1
      && JSValueGetType (js_context, js_arguments[0]) == kJSTypeBoolean)
  {
      bool sensitive = JSValueToBoolean (js_context, js_arguments[0]);
      if (sensitive == true)
      {
        gtk_window_set_icon_name (GTK_WINDOW (window), "face-smile-big");
      }
  }

  return js_value;
}

int
main (int     argc,
      gchar  *argv[])
{
  GtkWidget *scrolled;
  GtkWidget *toolbutton;
  GtkWidget *zoom;

  WebKitWebView     *web_view;
  WebKitWebFrame    *web_frame;

  GtkBuilder *ui;

  GError *error = NULL;

  if (!g_thread_supported ())
    g_thread_init (NULL);
  gtk_init_check (&argc, &argv);

  ui = gtk_builder_new ();

  gtk_builder_add_from_file (ui, UI_FILE, &error);
  gtk_builder_connect_signals (ui, NULL);

  if (error)
  {
    g_debug ("oops! %s", error->message);
    g_error_free (error);
    return 0;
  }

  window = GTK_WIDGET (gtk_builder_get_object (ui, "window"));
  scrolled = GTK_WIDGET (gtk_builder_get_object (ui, "scrolledwindow"));

  web_view = WEBKIT_WEB_VIEW (webkit_web_view_new ());
  webkit_web_view_load_uri (web_view, FILE_START);

  g_signal_connect (G_OBJECT (web_view), "create-plugin-widget",
                    G_CALLBACK (web_view_create_plugin_widget_cb), NULL);

  web_frame = webkit_web_view_get_main_frame (web_view);

  //webkit_web_view_set_full_content_zoom (web_view, TRUE);

  gtk_container_add (GTK_CONTAINER (scrolled), GTK_WIDGET (web_view));

  /* toolbuttons */
  toolbutton = GTK_WIDGET (gtk_builder_get_object (ui, "homebutton"));
  g_signal_connect (G_OBJECT (toolbutton), "clicked",
                    G_CALLBACK (home_clicked_cb), web_view);

  toolbutton = GTK_WIDGET (gtk_builder_get_object (ui, "quitbutton"));
  g_signal_connect (G_OBJECT (toolbutton), "clicked",
                    G_CALLBACK (quit_clicked_cb), web_view);

  /* zoom control */
  zoom = GTK_WIDGET (gtk_builder_get_object (ui, "zoomscale"));
  gtk_range_set_range (GTK_RANGE (zoom), 0.1, 1.5);
  gtk_range_set_value (GTK_RANGE (zoom), 1.0);
  g_signal_connect (G_OBJECT (zoom), "value-changed",
                    G_CALLBACK (zoomscale_value_changed_cb), web_view);

  g_signal_connect (G_OBJECT (window), "delete-event",
                    G_CALLBACK (window_delete_event_cb), NULL);

  /* js magic */
  JSGlobalContextRef js_context;
  JSObjectRef js_global;
  JSStringRef js_function_name;
  JSObjectRef js_set_can_register;

  js_context = webkit_web_frame_get_global_context (web_frame);
  js_global = JSContextGetGlobalObject (js_context);

  /* give a function name, register a C-callback */
  js_function_name = JSStringCreateWithUTF8CString ("holaEncuentro");
  js_set_can_register = JSObjectMakeFunctionWithCallback (js_context, js_function_name, (JSObjectCallAsFunctionCallback) hola_encuentro_cb);

  /* add us to the global context */
  JSObjectSetProperty (js_context, js_global, js_function_name, js_set_can_register, 0, NULL);
  JSStringRelease (js_function_name);

  gtk_widget_show_all (window);
  gtk_main ();

  return 0;
}
